@@ -3,7 +3,7 @@ package Ryu::Async;
 use strict;
 use warnings;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 =head1 NAME
 
@@ -43,9 +43,12 @@ This is an L<IO::Async::Notifier> subclass for interacting with L<Ryu>.
 use parent qw(IO::Async::Notifier);
 
 use IO::Async::Timer::Periodic;
+use IO::Async::Stream;
 use Ryu::Source;
 use curry::weak;
 
+use Log::Any qw($log);
+
 =head1 METHODS
 
 =cut
@@ -70,29 +73,16 @@ sub from {
     use namespace::clean qw(blessed weaken);
     my $self = shift;
 
-    my $src = $self->source(label => 'from');
     if(my $class = blessed $_[0]) {
         if($class->isa('IO::Async::Stream')) {
-            my $stream = shift;
-            $stream->configure(
-                on_read => sub {
-                    my ($stream, $buffref, $eof) = @_;
-                    my $data = substr $$buffref, 0, length $$buffref, '';
-                    $src->emit($data);
-                    $src->finish if $eof && !$src->completed->is_ready;
-                }
-            );
-            unless($stream->parent) {
-                $self->add_child($stream);
-                $src->completed->on_ready(sub {
-                    $self->remove_child($stream) if $stream->parent;
-                });
-            }
-            return $src;
+            return $self->from_stream($_[0]);
         } else {
             die "Unable to determine appropriate source for $class";
         }
-    } elsif(my $ref = ref $_[0]) {
+    }
+
+    my $src = $self->source(label => 'from');
+    if(my $ref = ref $_[0]) {
         if($ref eq 'ARRAY') {
             my @pending = @{$_[0]};
             my $code;
@@ -101,7 +91,8 @@ sub from {
                     $src->emit(shift @pending);
                     $self->loop->later($code);
                 } else {
-                    weaken($code);
+                    $src->finish;
+                    weaken($_) for $src, $code, $self;
                 }
             };
             $code->();
@@ -131,6 +122,57 @@ sub from {
     die "unknown stuff";
 }
 
+=head2 from_stream
+
+Create a new L<Ryu::Source> from an L<IO::Async::Stream> instance.
+
+Note that a stream which is not already attached to an L<IO::Async::Notifier>
+will be added as a child of this instance.
+
+=cut
+
+sub from_stream {
+    use Scalar::Util qw(blessed weaken);
+    use namespace::clean qw(blessed weaken);
+    my ($self, $stream) = @_;
+
+    my $src = $self->source(label => 'from');
+    $stream->configure(
+        on_read => sub {
+            my ($stream, $buffref, $eof) = @_;
+            $log->tracef("Have %d bytes of data, EOF = %s", length($$buffref), $eof ? 'yes' : 'no');
+            my $data = substr $$buffref, 0, length $$buffref, '';
+            $src->emit($data);
+            $src->finish if $eof && !$src->completed->is_ready;
+        }
+    );
+    unless($stream->parent) {
+        $self->add_child($stream);
+        $src->completed->on_ready(sub {
+            $self->remove_child($stream) if $stream->parent;
+        });
+    }
+    return $src;
+}
+
+=head2 from_stream
+
+Create a new L<Ryu::Source> that wraps STDIN.
+
+As with other L<IO::Async::Stream> wrappers, this will emit data as soon as it's available,
+as raw bytes.
+
+Use L<Ryu::Source/by_line> and L<Ryu::Source/decode> to split into lines and/or decode from UTF-8.
+
+=cut
+
+sub stdin {
+    my ($self) = @_;
+    return $self->from_stream(
+        IO::Async::Stream->new_for_stdin
+    )
+}
+
 =head2 timer
 
 Provides a L<Ryu::Source> which emits an empty string at selected intervals.
@@ -156,13 +198,11 @@ Example:
 sub timer {
     my ($self, %args) = @_;
     my $src = $self->source(label => 'timer');
-    my $code = $src->curry::weak::emit('');
     $self->add_child(
         my $timer = IO::Async::Timer::Periodic->new(
+            reschedule => 'hard',
             %args,
-            on_tick => sub {
-                $code->();
-            },
+            on_tick => $src->curry::weak::emit(''),
         )
     );
     Scalar::Util::weaken($timer);
@@ -184,9 +224,11 @@ Returns a new L<Ryu::Source> instance.
 
 sub source {
     my ($self, %args) = @_;
-    my $label = delete $args{label};
-    $label //= (caller 1)[3];
+    my $label = delete($args{label}) // (caller 1)[3];
     $label =~ s/^Net::Async::/Na/g;
+    $label =~ s/^IO::Async::/Ia/g;
+    $label =~ s/^Web::Async::/Wa/g;
+    $label =~ s/^Tickit::Async::/Ta/g;
     $label =~ s/::([^:]*)$/->$1/;
     Ryu::Source->new(
         new_future => $self->loop->curry::weak::new_future,
